#include "gui/loot_ui.hpp"
#include "constants.hpp"
#include <fmt/xchar.h>
#include "gui/guecstra.hpp"

namespace gui {
  using namespace guecs;

  LootUI::LootUI(GameLevel level) :
      $level(level)
  {
    $gui.position(RAY_VIEW_X+RAY_VIEW_WIDTH/2-200,
        RAY_VIEW_Y+RAY_VIEW_HEIGHT/2-200, 400, 400);

    $gui.layout(
        "[item_0 | item_1|item_2 | item_3]"
        "[item_4 | item_5|item_6 | item_7]"
        "[item_8 | item_9|item_10 | item_11]"
        "[button_12|button_13|button_14|button_15]"
        "[_ | *%(200)close|_ | _]"
        );
  }

  void LootUI::init() {
    using guecs::THEME;
    auto bg_color = THEME.DARK_LIGHT;
    bg_color.a = 140;
    $gui.set<Background>($gui.MAIN, {$gui.$parser, bg_color});

    // fill in 4 slots for prototype
    for(int i = 0; i < 4; i++) {
      auto id = $gui.entity("item_", i);

      $gui.set<guecs::Rectangle>(id, {THEME.PADDING,
          THEME.TRANSPARENT, THEME.LIGHT_MID });

      $gui.set<guecs::Effect>(id, {0.4f, "ui_shader"});
      $gui.set<guecs::Clickable>(id, {
          [=](auto, auto) { fmt::println("clicked button_{}", i); }
      });
      $gui.set<guecs::Sprite>(id, {"broken_yoyo-64"});
    }

    auto close = $gui.entity("close");
    $gui.set<guecs::Rectangle>(close, {});
    $gui.set<guecs::Label>(close, {L"CLOSE"});
    $gui.set<guecs::Clickable>(close,
        guecs::make_action(*$level.world, Events::GUI::LOOT_CLOSE));

    $gui.init();
  }

  void LootUI::render(sf::RenderWindow& window) {
    $gui.render(window);
  }

  void LootUI::update_level(GameLevel &level) {
    $level = level;
    init();
  }

  bool LootUI::mouse(float x, float y, bool hover) {
    return $gui.mouse(x, y, hover);
  }
}
